﻿/*
 * Ejection Seat
 * Author: Pandassaurus
 * Version: 0.1
 * Donate if you enjoy!
*/

using GTA;
using System;
using System.IO;
using System.Windows.Forms;
using GTA.Native;
using GTA.Math;

public static class UTILS
{
    //Thanks to libertylocked for this, as well as the prop and teleport functions!
    //Check out his fishing mod: https://www.gta5-mods.com/scripts/fishing-mod-libertylocked
    public static int PedBoneIndex(this Ped p, int b)
    {
        return Function.Call<int>(Hash.GET_PED_BONE_INDEX, p.Handle, b);
    }
}

public class EjectionSeat : Script
{
    //TODO: Controller Support, Better Launching(way you're facing, not just up), add sounds, fix ejector seat prop, be able to shoot while going up

    private Keys keyEject = Keys.Add; //default Add
    private Keys keyExplode = Keys.E; //default E
    private bool uCountdown = false; //default false
    private bool uRealism = true; //default true
    private bool uFireTrail = true; //default true
    private bool uEjectorSeatProp = false; //default false
    private float uPushForce = 8; //default 8

    private String version = "v0.2";
    private bool hasBeenPlugged = false;


    bool giveParachute = false;
    bool blowUpVehicle = false;
    private Prop rocket = null;
    private Prop ejectorSeatProp = null;

    public EjectionSeat()
    {
        Tick += OnTick;
        KeyUp += OnKeyUp;
        GetBindings();
    }

    private void GetBindings()
    {
        string getSettings = File.ReadAllText(AppDomain.CurrentDomain.BaseDirectory + "\\EjectionSeatSettings.txt");
        string[] enteredValues = getSettings.Split('=');

        string[] ejectKeyValue = enteredValues[2].Split('\n');
        keyEject = (Keys) Enum.Parse(typeof (Keys), ejectKeyValue[0]);

        string[] explodeKeyValue = enteredValues[3].Split('\n');
        keyExplode = (Keys) Enum.Parse(typeof (Keys), explodeKeyValue[0]);

        string[] countdownValue = enteredValues[4].Split('\n');
        uCountdown = Convert.ToBoolean(countdownValue[0]);

        string[] realismValue = enteredValues[5].Split('\n');
        uRealism = Convert.ToBoolean(realismValue[0]);

        string[] fireTrailValue = enteredValues[6].Split('\n');
        uFireTrail = Convert.ToBoolean(fireTrailValue[0]);

        string[] ejectorSeatPropValue = enteredValues[7].Split('\n');
        uEjectorSeatProp = Convert.ToBoolean(ejectorSeatPropValue[0]);

        string[] pushForceValue = enteredValues[8].Split('\n');
        uPushForce = Convert.ToInt32(pushForceValue[0]);
    }

    void OnTick(object sender, EventArgs e)
    {
        Ped playerPed = Game.Player.Character;     
 
        //When character is falling from launch
        if (Function.Call<bool>(Hash.IS_PED_FALLING, playerPed) && giveParachute)
        {
            //Stop Animation, give parachute, delete rocket
            playerPed.Task.ClearAllImmediately();
            playerPed.Task.PutAwayParachute();
            rocket.Delete();
            if(ejectorSeatProp != null) ejectorSeatProp.Delete();
            giveParachute = false;
            if (blowUpVehicle) UI.ShowSubtitle("");
            blowUpVehicle = false;

            //Remove entity proofs
            Function.Call(Hash.SET_ENTITY_PROOFS, playerPed, false, false, false, false, false, false, false, false);
        }
    }

    void OnKeyUp(object sender, KeyEventArgs e)
    {
        Ped pedPlayer = Game.Player.Character;

        if (e.KeyCode == keyEject)
        {
            if (pedPlayer.IsInVehicle() && !Function.Call<bool>(Hash.IS_PAUSE_MENU_ACTIVE))
            {
                //Name of the mod
                if (Game.Player.IsPlaying && !hasBeenPlugged)
                {
                    GTA.UI.Notify("Ejection Seat " + version + "\nBy Pandassaurus");
                    hasBeenPlugged = true;
                }

                Vehicle vehicle = pedPlayer.CurrentVehicle;
                String vDName = vehicle.DisplayName;

                //Convertibles
                if (vehicle.IsConvertible)
                {
                    //Open Roof
                    //Function.Call(Hash.SET_ENTITY_ANIM_SPEED, vehicle)
                    vehicle.RoofState = VehicleRoofState.Opened;
                    GTA.UI.ShowSubtitle("Initiating Sequence...", 100000);

                    //Wait to open
                    while (vehicle.RoofState != VehicleRoofState.Opened)
                    {
                        vehicle.RoofState = VehicleRoofState.Opened;
                        if (!pedPlayer.IsSittingInVehicle())
                        {
                            GTA.UI.ShowSubtitle("Sequence Aborted");
                            return;
                        }
                        Script.Wait(10);
                    }

                    //Launch!
                    //veh@plane@lazer@exit_skydive -> fds_jump_out_skydive
                    if (!EjectPlayer(vehicle, "amb@code_human_in_car_idles@generic@ds@idle_b", "idle_d"))
                    {
                        GTA.UI.ShowSubtitle("Sequence Aborted.");
                    }
                }

                //Planes
                else if (vDName == "DUSTER" || vDName == "BESRA" || vDName == "HYDRA" || vDName == "LAZER" ||
                         vDName == "STUNT")
                {
                    if (!EjectPlayer(vehicle, "amb@code_human_in_car_idles@generic@ds@idle_b", "idle_d"))
                    {
                        GTA.UI.ShowSubtitle("Sequence Aborted.");
                    }
                }

                //Topless Cars
                else if (!vehicle.HasRoof)
                {
                    if (!EjectPlayer(vehicle, "amb@code_human_in_car_idles@generic@ds@idle_b", "idle_d"))
                    {
                        GTA.UI.ShowSubtitle("Sequence Aborted.");
                    }
                }

                //Bikes and motorbikes and quadbikes
                else if (vehicle.Model.IsBike || vehicle.Model.IsBicycle || vehicle.Model.IsQuadbike)
                {
                    if (!EjectPlayer(vehicle, "veh@bike@chopper@front@base", "lean_r_slow_extreme"))
                    {
                        GTA.UI.ShowSubtitle("Sequence Aborted.");
                    }
                }

                //Incompatable cars
                else
                {
                    if (uRealism)
                    {
                        GTA.UI.ShowSubtitle("You must be in a vehicle without a roof.");
                    }
                    else
                    {
                        if (!EjectPlayer(vehicle, "amb@code_human_in_car_idles@generic@ds@idle_b", "idle_d"))
                        {
                            GTA.UI.ShowSubtitle("Sequence Aborted.");
                        }
                    }
                }
            }
        }


        //Explode Vehicle
        if (e.KeyCode == keyExplode && blowUpVehicle)
        {           
            World.AddExplosion(Game.Player.LastVehicle.Position, ExplosionType.ExplosionWithFire1, 200, 0.4f);
            Game.Player.LastVehicle.Explode();
            UI.ShowSubtitle("");
            blowUpVehicle = false;
        }
    }

    bool EjectPlayer(Vehicle vehicle, String animSet, String animName)
    {
        Ped playerPed = Game.Player.Character;

        //CountDown or no countdown
        if (uCountdown)
        {
            GTA.UI.ShowSubtitle("Launching in 3...", 500);
            for (int x = 0; x < 25; x++)
            {
                if (!playerPed.IsSittingInVehicle()) return false;
                Script.Wait(20);
            }
            GTA.UI.ShowSubtitle("Launching in 2...", 500);
            for (int x = 0; x < 25; x++)
            {
                if (!playerPed.IsSittingInVehicle()) return false;
                Script.Wait(20);
            }
            GTA.UI.ShowSubtitle("Launching in 1...", 500);
            for (int x = 0; x < 25; x++)
            {
                if (!playerPed.IsSittingInVehicle() && x < 22) return false;
                if (x == 23 && vehicle.Model.IsPlane) vehicle.OpenDoor(VehicleDoor.FrontLeftDoor, false, true);
                Script.Wait(20);
            }
            GTA.UI.ShowSubtitle("Enjoy your flight", 300);
        }
        else
        {
            if (vehicle.Model.IsPlane) vehicle.OpenDoor(VehicleDoor.FrontLeftDoor, false, true);
            Script.Wait(20);
        }


        //Teleport, set up blow up car       
        Vector3 playerInCarCoords = Function.Call<Vector3>(Hash.GET_ENTITY_COORDS, playerPed.Handle, true);
        Function.Call(Hash.SET_ENTITY_COORDS, playerPed.Handle, playerInCarCoords.X + playerPed.UpVector.X, playerInCarCoords.Y + playerPed.UpVector.Y, 
            playerInCarCoords.Z + playerPed.UpVector.Z, true, true, true, 1);
        blowUpVehicle = true;
        GTA.UI.ShowSubtitle("Press \"" + keyExplode.ToString() + "\" to blow up your vehicle.", 10000);
        
        //Let player live through this and Anim
        playerPed.CanRagdoll = false;
        Function.Call(Hash.SET_ENTITY_PROOFS, playerPed, false, true, true, false, false, false, false, false);
        playerPed.Task.PlayAnimation(animSet, animName, 24f, -1, true, 32f);

        //Ejector seat attatch. If being realistic, only use chair for cars and planes
        if(ejectorSeatProp != null) ejectorSeatProp.Delete();
        if (uEjectorSeatProp && uRealism)
        {
            if (vehicle.Model.IsCar || vehicle.Model.IsPlane)
            {
                ejectorSeatProp = World.CreateProp(new Model("prop_ejector_seat_01"), Vector3.Zero, false, false);
                ejectorSeatProp.IsInvincible = true;
                ejectorSeatProp.AttachTo(playerPed, playerPed.PedBoneIndex(Bone.SKEL_Pelvis.GetHashCode()), new Vector3(.09f, -0.01f, 0f), new Vector3(0f, 90f, 165f));
            }
        }

        else if (uEjectorSeatProp)
        {
            ejectorSeatProp = World.CreateProp(new Model("prop_ejector_seat_01"), Vector3.Zero, false, false);
            ejectorSeatProp.IsInvincible = true;
            ejectorSeatProp.AttachTo(playerPed, playerPed.PedBoneIndex(Bone.SKEL_Pelvis.GetHashCode()), new Vector3(.09f, -0.01f, 0f), new Vector3(0f, 90f, 165f));
        }

        //Rocket Attatch
        if(rocket != null) rocket.Delete();
        rocket = World.CreateProp(new Model("w_lr_homing_rocket"), Vector3.Zero, false, false);
        rocket.AttachTo(playerPed, playerPed.PedBoneIndex(Bone.SKEL_Pelvis.GetHashCode()), new Vector3(0.25f, -0.05f, 0f), new Vector3(0f, 0f, 70f));   

        //Gradual Launch with FIRE, then allow ragdoll and give parachute
        for (int i = 0; i < 156; i++)
        {
            if (!Function.Call<bool>(Hash.IS_PED_FALLING, playerPed))
            {
                if (i == 2 || i == 6 || i == 14 || i == 24 || i == 36 || i == 52 || i == 72 || i == 96 || i == 124 ||
                    i == 156)
                {
                    playerPed.ApplyForce(new Vector3(0f, 0f, uPushForce));
                }
                if(i > 10 && i < 140 && uFireTrail) World.AddExplosion(rocket.Position + new Vector3(0, 0, .08f), ExplosionType.ValveFire2, 2f, 6f);
                Script.Wait(1);
            }
            else
            {
                break;
            }
        }
        playerPed.CanRagdoll = true;
        giveParachute = true;
        return true;
    }
}